#ifndef SRC_HDF5_BODYWRITER_HH
#define SRC_HDF5_BODYWRITER_HH

#include <dune/fufem/functions/basisgridfunction.hh>
#include <dune/fufem/geometry/convexpolyhedron.hh>
#include <dune/fufem/hdf5/file.hh>

#include "hdf5/frictionalboundary-writer.hh"
#include "hdf5/patchinfo-writer.hh"

template <class ProgramState, class VertexBasis, class GridView>
class HDF5BodyWriter {
private:
    using PatchInfoWriter = PatchInfoWriter<ProgramState, VertexBasis, GridView>;
    using FrictionalBoundaryWriter = FrictionalBoundaryWriter<ProgramState, GridView>;
public:
    using VertexCoordinates = typename ProgramState::Vector;
    using FrictionPatches = std::vector<const BoundaryPatch<GridView>* >;
    using WeakPatches = std::vector<const ConvexPolyhedron<LocalVector>* >;

    using LocalVector = typename VertexCoordinates::block_type;

    //friend class HDF5LevelWriter<ProgramState, VertexBasis, GridView>;

    HDF5BodyWriter(
            HDF5::Grouplike &file,
            const VertexCoordinates& vertexCoordinates,
            const VertexBasis& vertexBasis,
            const FrictionPatches& frictionPatches,
            const WeakPatches& weakPatches) :
        patchCount_(frictionPatches.size()),
        file_(file),
#if MY_DIM == 3
        patchInfoWriters_(patchCount_),
#endif
        frictionBoundaryWriters_(patchCount_) {

        assert(patchCount_ == weakPatches.size());

        for (size_t i=0; i<patchCount_; i++) {
#if MY_DIM == 3
            patchInfoWriters_[i] = std::make_unique<PatchInfoWriter>(file_, vertexBasis, *frictionPatches[i], *weakPatches[i], i);
#endif
            frictionBoundaryWriters_[i] = std::make_unique<FrictionalBoundaryWriter>(file_, vertexCoordinates, *frictionPatches[i], i);
        }
    }

    template <class Friction>
    void reportSolution(ProgramState const &programState,
                      // for the friction coefficient
                      Friction& friction) {

        for (size_t i=0; i<patchCount_; i++) {
#if MY_DIM == 3
            patchInfoWriters_[i]->write(programState);
#endif
            frictionBoundaryWriters_[i]->write(programState, friction);
        }
    }

private:
    const size_t patchCount_;

    HDF5::Grouplike &file_;

#if MY_DIM == 3
    std::vector<std::unique_ptr<PatchInfoWriter>> patchInfoWriters_;
#endif

    std::vector<std::unique_ptr<FrictionalBoundaryWriter>> frictionBoundaryWriters_;
};
#endif
